#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from time import sleep

import pandas as pd
from catalyst.constants import LOG_LEVEL
from catalyst.gens.sim_engine import (
    BAR,
    SESSION_START,
    SESSION_END,
)
from logbook import Logger

log = Logger('ExchangeClock', level=LOG_LEVEL)


class SimpleClock(object):
    """Realtime clock for live trading.

    This class is a drop-in replacement for
    :class:`zipline.gens.sim_engine.MinuteSimulationClock`.

    This is a stripped down version because crypto exchanges run
    around the clock.

    The :param:`time_skew` parameter represents the time difference between
    the Broker and the live trading machine's clock.
    """

    def __init__(self, sessions, time_skew=pd.Timedelta("0s"), start=None,
                 end=None):

        self.sessions = sessions
        self.time_skew = time_skew
        self._last_emit = None
        self._before_trading_start_bar_yielded = True
        self.start = start
        self.end = end

    def __iter__(self):
        self.handle_late_start()
        yield pd.Timestamp.utcnow(), SESSION_START

        while True:
            current_time = pd.Timestamp.utcnow()
            current_minute = current_time.floor('1 min')

            if self.end is not None and current_minute >= self.end:
                break
            if self._last_emit is None or current_minute > self._last_emit:
                log.debug('emitting minutely bar: {}'.format(current_minute))

                self._last_emit = current_minute
                yield current_minute, BAR
            else:
                sleep(1)

        yield current_minute, SESSION_END

    def handle_late_start(self):
        if self.start:
            time_diff = (self.start - pd.Timestamp.utcnow())
            log.info(
                'The algorithm is waiting for the specified '
                'start date: {}'.format(self.start))
            sleep(time_diff.seconds)

            while pd.Timestamp.utcnow() < self.start:
                pass



from catalyst.api import order, record, symbol
from catalyst.utils.run_algo import run_algorithm
import pandas as pd

def initialize(context):
    pass




def handle_data(context, data):
    current_price = data.current(symbol('eth_usd'), fields=['price'])
    current_open = data.current(symbol('eth_usd'), fields=['open'])


    #prices = data.history(
    #    symbol('btc_eur'),
    #    fields=['price', 'open', 'high', 'low', 'close', 'volume'],
    #    bar_count=3,
    #    frequency="5T"
    #)

    #record(
    #    volume=current['volume'], prices = prices
    #)
    print ("CURRENT: %s %s %s" % (data.current_dt, current_price, current_open))


#LIVE
run_algorithm(
    initialize=initialize,
    exchange_name='bitfinex', # also tried poloniex
    quote_currency='usd',# or usdt
    capital_base=20000,
    algo_namespace="issue-230",
    data_frequency='minute',
    handle_data=handle_data,
    start=pd.to_datetime('2018-2-27', utc=True),
    end=pd.to_datetime('2018-3-1', utc=True)) # False also
    #live=True,
    #simulate_orders=True) # False also





import pandas as pd
import datetime
import sys

def read_csv(csv_filename, ascending=True):
    df = pd.read_csv(csv_filename)
    df['last_traded'] = pd.to_datetime(df['last_traded'])
    #df.set_index(
    #    'last_traded', inplace=True, drop=True
    #)
    #df.index = df.index.tz_localize('UTC')
    df.sort_index(inplace=True, ascending=False)

    return df


if __name__ == '__main__':
    if len(sys.argv) >= 2:
        file_name = sys.argv[1]
    else:
        "no file was specified"

    df = read_csv(file_name)

    df['gap'] = df['last_traded'].diff().iloc[1:] > datetime.timedelta(minutes=1)

    print df[df['gap'] == True]

    """
    for i in xrange(len(df)-1):
        if df['last_traded'][i] - df['last_traded'][i+1] >= datetime.timedelta(minutes=1):
            print ("found gap at %s %s" % (df['last_traded'][i+1], df['last_traded'][i]))
    """



from datetime import timedelta

import pandas as pd

from catalyst import run_algorithm
from catalyst.api import (symbols, )
from catalyst.exchange.utils.exchange_utils import get_exchange_symbols


def initialize(context):
    context.exchange = 'poloniex'
    context.base_currency = 'btc'

def handle_data(context, data):
    # run the following every so often in live mode to get the universe of active coins
    # in the given exchange quoted in base_currency
    context.coins = context.exchanges[context.exchange].assets
    context.coins = [c for c in context.coins
                             if c.quote_currency == context.base_currency]

#LIVE
run_algorithm(capital_base=0.07211586,  # amount of base_currency
            initialize=initialize,
            handle_data=handle_data,
            output='out_big_v2_live.pickle',
            exchange_name='poloniex',
            data_frequency='minute',
            base_currency='btc',
            live=True,
            simulate_orders=True,
            live_graph=False,
            algo_namespace='big_v2_live')



import os
import csv
import pytz
from datetime import datetime

from catalyst.api import record, symbol, symbols
from catalyst.utils.run_algo import run_algorithm


def initialize(context):
    # Portfolio assets list
    context.asset = symbol('btc_usdt') # Bitcoin on Poloniex

    # Creates a .CSV file with the same name as this script to store results
    context.csvfile = open(os.path.splitext(os.path.basename('btc_usdt_error'))[0]+'.csv', 'w+')
    context.csvwriter = csv.writer(context.csvfile)
    context.csvwriter.writerow(['date', 'open', 'close', 'high', 'low', 'volume'])

def handle_data(context, data):
    # Variables to record for a given asset: price and volume
    # Other options include 'open', 'high', 'open', 'close'
    # Please note that 'price' equals 'close'
    date   = context.blotter.current_dt     # current time in each iteration
    opened = data.current(context.asset, 'open')
    close = data.current(context.asset, 'close')
    high = data.current(context.asset, 'high')
    low = data.current(context.asset, 'low')
    volume = data.current(context.asset, 'volume')

    # Writes one line to CSV on each iteration with the chosen variables
    context.csvwriter.writerow([date, opened, close, high, low, volume])

def analyze(context=None, results=None):
    # Close open file properly at the end
    context.csvfile.close()


# Bitcoin data is available from 2015-3-2. Dates vary for other tokens.
start = datetime(2017, 12, 6, 0, 0, 0, 0, pytz.utc)
end = datetime(2017, 12, 8, 0, 0, 0, 0, pytz.utc)
results = run_algorithm(
    initialize=initialize,
    handle_data=handle_data,
    analyze=analyze,
    start=start,
    end=end,
    exchange_name='poloniex',
    data_frequency='minute',
    base_currency ='usdt',
    capital_base=10000)


from catalyst.api import order, record, symbol
from catalyst.utils.run_algo import run_algorithm
import pandas as pd

def initialize(context):
    pass




def handle_data(context, data):
    current = data.current(symbol('btc_eur'), fields=['volume'])


    prices = data.history(
        symbol('eth_usdt'),
        fields=['price', 'open', 'high', 'low', 'close', 'volume'],
        bar_count=3,
        frequency="5T"
    )

    #record(
    #    volume=current['volume'], prices = prices
    #)
    print ("CURRENT VOLUME: %s" % prices)


#LIVE
run_algorithm(
    initialize=initialize,
    exchange_name='bitfinex', # also tried poloniex
    base_currency='eur',# or usdt
    capital_base=20000,
    algo_namespace="issue-230",
    handle_data=handle_data,
    start=pd.to_datetime('2018-2-11', utc=True),
    end=pd.to_datetime('2018-2-12', utc=True)) # False also
    #live=True,
    #simulate_orders=True) # False also


"""

#BACK-TEST
run_algorithm(
    initialize=initialize,
    exchange_name='poloniex', # also tried poloniex
    base_currency='usdt',# or usdt
    capital_base=20000,
    data_frequency='minute',
    algo_namespace="issue-230",
    handle_data=handle_data,
    start=pd.to_datetime('2018-2-11', utc=True),
    end=pd.to_datetime('2018-2-12', utc=True)) # False also



run_algorithm(
                capital_base=100,
                data_frequency='minute',
                initialize=initialize,
                handle_data=handle_data,
                analyze=analyze,
                exchange_name='poloniex',
                algo_namespace=NAMESPACE,
                base_currency='usdt',
                start=pd.to_datetime('2018-2-12', utc=True),
                end=pd.to_datetime('2018-2-12', utc=True),
                output=out
            )

And I am using this configuration to run live against Poloniex:

        if live:
            run_algorithm(
                capital_base=100,
                initialize=initialize,
                handle_data=handle_data,
                analyze=analyze,
                exchange_name='poloniex',
                live=True,
                algo_namespace=NAMESPACE,
                base_currency='usdt',
                live_graph=False,
                simulate_orders=False,
                stats_output=None,
            )

The only thing I change to backtest is using this configuration:

run_algorithm(
                capital_base=100,
                data_frequency='minute',
                initialize=initialize,
                handle_data=handle_data,
                analyze=analyze,
                exchange_name='poloniex',
                algo_namespace=NAMESPACE,
                base_currency='usdt',
                start=pd.to_datetime('2018-2-12', utc=True),
                end=pd.to_datetime('2018-2-12', utc=True),
                output=out
            )
"""


from catalyst.api import order, record, symbol, order_target_percent
from catalyst.api import order, record, symbol
from catalyst.utils.run_algo import run_algorithm

def initialize(context):
    context.asset = symbol('btc_usd')
    context.i = 0
    #context.set_slippage(spread=0.01)

def handle_data(context, data):
    context.i += 1

    print("========")
    print(context.i)
    print("========")

    if context.i == 1:
        order(context.asset, 10)
        #order_target_percent(context.asset, 1)

    elif context.i == 2:
        order(context.asset, -10)
        #order_target_percent(context.asset, -1)

    record(data.current(context.asset, 'price'))



#LIVE
run_algorithm(
    initialize=initialize,
    exchange_name='bitmex', # also tried poloniex
    base_currency='usd',# or usdt
    capital_base=0,
    algo_namespace="bitmex",
    handle_data=handle_data,
    #start=pd.to_datetime('2017-1-1', utc=True),
    #end=pd.to_datetime('2018-4-8', utc=True), # False also
    live=True,
    simulate_orders=False) # False also



import pandas as pd
from catalyst.utils.run_algo import run_algorithm
from catalyst.api import symbol
from datetime import datetime
import pytz

def initialize(context):
    context.i = 0

def handle_data(context, data):
    prices = data.history(symbol('btc_usdt'), fields=['close'],bar_count=1,frequency='1T')
    context.i = context.i + 1
    #if context.i == 3: context.interrupt_algorithm()
    print prices

start_end = datetime(2018, 1, 1, 0, 0, 0, 0, pytz.utc)
run_algorithm(
    capital_base=3000,
    initialize=initialize,
    handle_data=handle_data,
    exchange_name='binance',
    algo_namespace='Test candles',
    quote_currency='usdt',
    data_frequency='minute',
    live=True,
    #start=start_end,
    #end=start_end
    )



from catalyst.api import symbol
from catalyst.utils.run_algo import run_algorithm

def initialize(context):
    pass


def handle_data(context, data):
    history = data.history(symbol('btc_usd'), ['volume'],
                           bar_count=288,
                           frequency='5T')

    print('\nnow: %s\n%s' % (data.current_dt, history))
    if not hasattr(context, 'i'):
        context.i = 0
    context.i += 1
    if context.i > 5:
        raise Exception('stop')


live = True
if live:
    run_algorithm(initialize=lambda ctx: True,
                  handle_data=handle_data,
                  exchange_name='gdax',
                  quote_currency='usd',
                  algo_namespace='issue-323',
                  live=True,
                  data_frequency='daily',
                  capital_base=3000,
                  )




from catalyst.api import symbol
from catalyst.utils.run_algo import run_algorithm

import pandas as pd
import numpy as np

#coins = ['btc', 'dash', 'etc', 'eth', 'ltc', 'nxt', 'rep', 'str', 'xmr', 'xrp', 'zec']
symbols = None
from catalyst.exchange.utils.factory import get_exchange

def initialize(context):
    pass


def _handle_data(context, data):
    global symbols
    #if symbols is None: symbols = [symbol(c + '_usd') for c in coins]

    lookback = 10

    exchange = get_exchange('bitfinex', skip_init=True)
    exchange.init()
    exchange_coins = []
    for asset in exchange.assets:
        print asset.asset_name
        #sim =  symbol(exchange.get_symbol(asset))

        #print sim
        exchange_coins.append(symbol(asset.asset_name))

    for coin in exchange_coins:
        print coin
        opened = fill(data.history(coin,
                                   'open',
                                   bar_count=lookback,
                                   frequency='1H')).values
        high = fill(data.history(coin,
                                 'high',
                                 bar_count=lookback,
                                 frequency='1H')).values
        low = fill(data.history(coin,
                                'low',
                                bar_count=lookback,
                                frequency='1H')).values
        close = fill(data.history(coin,
                                  'price',
                                  bar_count=lookback,
                                  frequency='1H')).values
        volume = fill(data.history(coin,
                                   'volume',
                                   bar_count=lookback,
                                   frequency='1H')).values

        # close[-1] is the last value in the set, which is the equivalent
        # to current price (as in the most recent value)
        # displays the minute price for each pair every 30 minutes
        print('{now}: {pair} -\tO:{o},\tH:{h},\tL:{c},\tC{c},'
              '\tV:{v}'.format(
            now=now,
            pair=pair,
            o=opened[-1],
            h=high[-1],
            l=low[-1],
            c=close[-1],
            v=volume[-1],
        ))

# Replace all NA, NAN or infinite values with its nearest value
def fill(series):
    if isinstance(series, pd.Series):
        return series.replace([np.inf, -np.inf], np.nan).ffill().bfill()
    elif isinstance(series, np.ndarray):
        return pd.Series(series).replace(
                     [np.inf, -np.inf], np.nan
                    ).ffill().bfill().values
    else:
        return series


run_algorithm(initialize=initialize,
              handle_data=_handle_data,
            analyze=lambda _, results: True,
            exchange_name='bitfinex',
            base_currency='usdt',
            algo_namespace='issue-simple-universe',
            live=True,
            data_frequency='minute',
            capital_base=3000,
            simulate_orders=True)


from catalyst.exchange.utils.factory import get_exchange

for exchange_name in ["gdax", "binance"]:
    exchange = get_exchange(exchange_name)
    assets = exchange.get_assets()
    print(exchange.tickers(assets[0:2]))
    print(exchange.tickers([assets[0]]))



import pytz
from datetime import datetime, timedelta
from catalyst.api import symbol
from catalyst.utils.run_algo import run_algorithm

coin = 'btc'
base_currency = 'usd'
n_candles = 5


def initialize(context):
    context.symbol = symbol('%s_%s' % (coin, base_currency))


def handle_data(context, data):
    # all I do is print the current last 2 candles (5T)
    history = data.history(symbol('eth_usdt'), ['volume'],
                           bar_count=1,
                           frequency='5T')

    current = data.current(symbol('eth_usdt'), ['price'])

    #print('\nnow: %s\n%s' % (data.current_dt, history))

    print('%s %s' % (data.current_dt, current))


live = True
if live:
    run_algorithm(initialize=lambda ctx: True,
                  handle_data=handle_data,
                  exchange_name='poloniex',
                  quote_currency='usd',
                  algo_namespace='issue-337',
                  live=True,
                  # data_frequency='minute',
                  capital_base=3000,

                  #start=datetime(2018, 3, 25, 0, 0, 0, 0, pytz.utc),
                  #end=datetime(2018, 4, 3, 0, 0, 0, 0, pytz.utc)
     )
else:
    run_algorithm(initialize=lambda ctx: True,
                  handle_data=handle_data,
                  exchange_name='bitfinex',
                  quote_currency='usd',
                  algo_namespace='issue-337',
                  live=False,
                  #data_frequency='minute',
                  capital_base=3000,
                  start=datetime(2018, 3, 25, 0, 0, 0, 0, pytz.utc),
                  end=datetime(2018, 4, 3, 0, 0, 0, 0, pytz.utc))


from catalyst.api import symbol, order
from catalyst.utils.run_algo import run_algorithm

def initialize(context):
    context.asset = symbol('abt_btc')


def handle_data(context, data):
    history = data.history(context.asset, ['volume'],
                           bar_count=1,
                           frequency='5T')

    print('\nnow: %s\n%s' % (data.current_dt, history))
    if not hasattr(context, 'i'):
        context.i = 0
    context.i += 1
    if context.i == 1:
        order(context.asset, 1000)
    if context.i > 10:
        raise Exception('stop')


live = True
if live:
    run_algorithm(initialize=initialize,
                  handle_data=handle_data,
                  exchange_name='huobipro',
                  quote_currency='usdt',
                  algo_namespace='issue-324',
                  live=True,
                  data_frequency='daily',
                  capital_base=3000,
                )




from catalyst.exchange.exchange_bundle import ExchangeBundle
import pandas as pd

exchange_name = "poloniex"

exchange_bundle = ExchangeBundle(exchange_name)

exchange_bundle.ingest(
    data_frequency="minute",
    include_symbols="btc_usdt",
    start=pd.Timestamp("01-01-2016"),
    end=pd.Timestamp("01-02-2017"),
)


from datetime import datetime
import pytz

from catalyst.api import symbol, cancel_order, order
from catalyst.utils.run_algo import run_algorithm
from catalyst.exchange.utils.exchange_utils import get_exchange_symbols

EXCHANGE_NAME = 'poloniex'

def initialize(context):
    context.asset = symbol('etc_usdt')
    context.exchange_name = EXCHANGE_NAME
    context.i = 0

def handle_data(context, data):
    json_symbols = get_exchange_symbols(context.exchange_name)
    #symbol()

    orders = context.blotter.open_orders
    if context.i == 0:
        order(context.asset, 1000, limit_price=0.001)

    if context.i == 0:
        for ord in context.blotter.open_orders[context.asset]:
            cancel_order(ord.id)#, EXCHANGE_NAME)
            #context.exchanges[EXCHANGE_NAME].cancel_order(ord.id)

    context.i += 1
    print json_symbols
    #context.exchanges[EXCHANGE_NAME].cancel_order(5)


live = False

if live:
    run_algorithm(initialize=initialize,
                  handle_data=handle_data,
                  exchange_name=EXCHANGE_NAME,
                  quote_currency='usdt',
                  algo_namespace='issue-342--',
                  live=True,
                  data_frequency='daily',
                  capital_base=104,
                  simulate_orders=True,
                )
else:
    run_algorithm(initialize=initialize,
                  handle_data=handle_data,
                  exchange_name=EXCHANGE_NAME,
                  quote_currency='usdt',
                  algo_namespace='issue-337',
                  live=False,
                  #data_frequency='minute',
                  capital_base=3000,
                  start=datetime(2018, 3, 25, 0, 0, 0, 0, pytz.utc),
                  end=datetime(2018, 4, 3, 0, 0, 0, 0, pytz.utc))


